var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

// Styles
import '../../../src/components/VColorPicker/VColorPicker.sass';
// Components
import VSheet from '../VSheet/VSheet';
import VColorPickerPreview from './VColorPickerPreview';
import VColorPickerCanvas from './VColorPickerCanvas';
import VColorPickerEdit, { modes } from './VColorPickerEdit';
import VColorPickerSwatches from './VColorPickerSwatches';
// Helpers
import { parseColor, fromRGBA, extractColor } from './util';
import mixins from '../../util/mixins';
import Themeable from '../../mixins/themeable';
export default mixins(Themeable).extend({
    name: 'v-color-picker',
    props: {
        canvasHeight: {
            type: [String, Number],
            default: 150
        },
        disabled: Boolean,
        dotSize: {
            type: [Number, String],
            default: 10
        },
        flat: Boolean,
        hideCanvas: Boolean,
        hideInputs: Boolean,
        hideModeSwitch: Boolean,
        mode: {
            type: String,
            default: 'rgba',
            validator: function validator(v) {
                return Object.keys(modes).includes(v);
            }
        },
        showSwatches: Boolean,
        swatches: Array,
        swatchesMaxHeight: {
            type: [Number, String],
            default: 150
        },
        value: {
            type: [Object, String]
        },
        width: {
            type: [Number, String],
            default: 300
        }
    },
    data: function data() {
        return {
            internalValue: fromRGBA({ r: 255, g: 0, b: 0, a: 1 })
        };
    },
    watch: {
        value: {
            handler: function handler(color) {
                this.updateColor(parseColor(color, this.internalValue));
            },

            immediate: true
        }
    },
    methods: {
        updateColor: function updateColor(color) {
            this.internalValue = color;
            var value = extractColor(this.internalValue, this.value);
            if (value !== this.value) {
                this.$emit('input', value);
                this.$emit('update:color', this.internalValue);
            }
        },
        genCanvas: function genCanvas() {
            return this.$createElement(VColorPickerCanvas, {
                props: {
                    color: this.internalValue,
                    disabled: this.disabled,
                    dotSize: this.dotSize,
                    width: this.width,
                    height: this.canvasHeight
                },
                on: {
                    'update:color': this.updateColor
                }
            });
        },
        genControls: function genControls() {
            return this.$createElement('div', {
                staticClass: 'v-color-picker__controls'
            }, [this.genPreview(), !this.hideInputs && this.genEdit()]);
        },
        genEdit: function genEdit() {
            var _this = this;

            return this.$createElement(VColorPickerEdit, {
                props: {
                    color: this.internalValue,
                    disabled: this.disabled,
                    hideModeSwitch: this.hideModeSwitch,
                    mode: this.mode
                },
                on: {
                    'update:color': this.updateColor,
                    'update:mode': function updateMode(v) {
                        return _this.$emit('update:mode', v);
                    }
                }
            });
        },
        genPreview: function genPreview() {
            return this.$createElement(VColorPickerPreview, {
                props: {
                    color: this.internalValue,
                    disabled: this.disabled
                },
                on: {
                    'update:color': this.updateColor
                }
            });
        },
        genSwatches: function genSwatches() {
            return this.$createElement(VColorPickerSwatches, {
                props: {
                    dark: this.dark,
                    light: this.light,
                    swatches: this.swatches,
                    color: this.internalValue,
                    maxHeight: this.swatchesMaxHeight
                },
                on: {
                    'update:color': this.updateColor
                }
            });
        }
    },
    render: function render(h) {
        return h(VSheet, {
            staticClass: 'v-color-picker',
            class: _extends({
                'v-color-picker--flat': this.flat
            }, this.themeClasses),
            props: {
                maxWidth: this.width
            }
        }, [!this.hideCanvas && this.genCanvas(), this.genControls(), this.showSwatches && this.genSwatches()]);
    }
});
//# sourceMappingURL=VColorPicker.js.map