var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

// Styles
import '../../../src/components/VTextarea/VTextarea.sass';
// Extensions
import VTextField from '../VTextField/VTextField';
// Utilities
import mixins from '../../util/mixins';
var baseMixins = mixins(VTextField);
/* @vue/component */
export default baseMixins.extend({
    name: 'v-textarea',
    props: {
        autoGrow: Boolean,
        noResize: Boolean,
        rowHeight: {
            type: [Number, String],
            default: 24,
            validator: function validator(v) {
                return !isNaN(parseFloat(v));
            }
        },
        rows: {
            type: [Number, String],
            default: 5,
            validator: function validator(v) {
                return !isNaN(parseInt(v, 10));
            }
        }
    },
    computed: {
        classes: function classes() {
            return _extends({
                'v-textarea': true,
                'v-textarea--auto-grow': this.autoGrow,
                'v-textarea--no-resize': this.noResizeHandle
            }, VTextField.options.computed.classes.call(this));
        },
        noResizeHandle: function noResizeHandle() {
            return this.noResize || this.autoGrow;
        }
    },
    watch: {
        lazyValue: function lazyValue() {
            this.autoGrow && this.$nextTick(this.calculateInputHeight);
        },
        rowHeight: function rowHeight() {
            this.autoGrow && this.$nextTick(this.calculateInputHeight);
        }
    },
    mounted: function mounted() {
        var _this = this;

        setTimeout(function () {
            _this.autoGrow && _this.calculateInputHeight();
        }, 0);
    },

    methods: {
        calculateInputHeight: function calculateInputHeight() {
            var input = this.$refs.input;
            if (!input) return;
            input.style.height = '0';
            var height = input.scrollHeight;
            var minHeight = parseInt(this.rows, 10) * parseFloat(this.rowHeight);
            // This has to be done ASAP, waiting for Vue
            // to update the DOM causes ugly layout jumping
            input.style.height = Math.max(minHeight, height) + 'px';
        },
        genInput: function genInput() {
            var input = VTextField.options.methods.genInput.call(this);
            input.tag = 'textarea';
            delete input.data.attrs.type;
            input.data.attrs.rows = this.rows;
            return input;
        },
        onInput: function onInput(e) {
            VTextField.options.methods.onInput.call(this, e);
            this.autoGrow && this.calculateInputHeight();
        },
        onKeyDown: function onKeyDown(e) {
            // Prevents closing of a
            // dialog when pressing
            // enter
            if (this.isFocused && e.keyCode === 13) {
                e.stopPropagation();
            }
            this.$emit('keydown', e);
        }
    }
});
//# sourceMappingURL=VTextarea.js.map